@@ -284,8 +284,8 @@ R_X<-R_X[-tie.indx,,drop=FALSE]
 R_RE_mat<-R_RE_mat[-tie.indx,,drop=FALSE]
 for(i in 1:length(tie.indx)){
 R_Y<-c(R_Y,1,0)
-R_X<-rBind(R_X,X.tie[i,,drop=FALSE],X.tie[i,,drop=FALSE])
-R_RE_mat<-rBind(R_RE_mat,Z.tie[i,,drop=FALSE],Z.tie[i,,drop=FALSE])
+R_X<-rbind(R_X,X.tie[i,,drop=FALSE],X.tie[i,,drop=FALSE])
+R_RE_mat<-rbind(R_RE_mat,Z.tie[i,,drop=FALSE],Z.tie[i,,drop=FALSE])
 }
 Nr <- length(R_Y)
 }
@@ -499,9 +499,9 @@ first.order.eblup<-cbind(1:length(teams),eblup[order(eblup[,1],decreasing=TRUE),
                               if (check.parmFE1 & check.parmFE2 | FE.count > 0) {
               if(indx.5==n_eta) cat("Calculating FE corrections for random effects covariance matrix...\n")
               FE.count <- FE.count + 1
-                    res.temp <- cBind(guide2, mapply(Trace.calc2, comp.k = guide2[, 1], comp.l = guide2[, 2], MoreArgs = list(dsig.dc.indx5, indx.5 = indx.5)))
+                    res.temp <- cbind(guide2, mapply(Trace.calc2, comp.k = guide2[, 1], comp.l = guide2[, 2], MoreArgs = list(dsig.dc.indx5, indx.5 = indx.5)))
                     temp.comp <- temp.comp[-(guide1), , drop = FALSE]
-                    var.corrections <- rBind(var.corrections, res.temp)
+                    var.corrections <- rbind(var.corrections, res.temp)
                     }
    
                   }
@@ -535,7 +535,7 @@ rbetasn <- update.rbetas.first.order(eta = eta, rbetas = rbetas, R_X=R_X, R_Y=R_
     var.eta <- var.eta + 0.5 * trc.y2
     rm(trc.y2)
     var.eta.hat <- var.eta
-    eblup <- as.matrix(cBind(eta.hat, sqrt(diag(var.eta.hat))))
+    eblup <- as.matrix(cbind(eta.hat, sqrt(diag(var.eta.hat))))
     colnames(eblup) <- c("eblup", "std. error")
     rownames(eblup) <- rep(teams,each=2)
     rm(var.eta)
@@ -578,7 +578,7 @@ rbetasn <- update.rbetas.first.order(eta = eta, rbetas = rbetas, R_X=R_X, R_Y=R_
      gt1<-gt1+temp_mat[(2*(i-1)+1):(2*i),(2*(i-1)+1):(2*i)]
     }
     gt1<-symmpart(gt1/length(teams))
-    Gn<-kronecker(Diagonal(length(teams)),gt1)
+    Gn<-as(kronecker(Diagonal(length(teams)),gt1),"generalMatrix")
 
 
 sigup<-(1/Ny)*(crossprod(Y-X%*%ybetas-Z%*%eta.hat)+sum(diag(crossprod(Z)%*%var.eta.hat)))
@@ -600,7 +600,7 @@ sigup<-(1/Ny)*(crossprod(Y-X%*%ybetas-Z%*%eta.hat)+sum(diag(crossprod(Z)%*%var.e
     it.time <- (proc.time() - ptm)[3]
     time.mat[it, ] <- c(it.time)
     cat("Iteration", it, "took", it.time, "\n")
-    eblup <- cBind(eta.hat, sqrt(diag(var.eta.hat)))
+    eblup <- cbind(eta.hat, sqrt(diag(var.eta.hat)))
 
     colnames(eblup) <- c("eblup", "std. error")
     rownames(eblup) <- rep(teams,each=2)
